package com.tfm.reservas.controller;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.Equipment;
import com.tfm.reservas.services.EquipmentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Equipment")
public class EquipmentController {

    @Autowired
    private EquipmentService equipmentService;

    @GetMapping("/equipments")
    @ApiOperation(value = "Cerca tots els Equipment", notes = "Retorna tots els Equipments")
    public List<Equipment> getAllEquipments() {
        return equipmentService.getAllEquipments();
    }

    @GetMapping("/equipments/{id}")
    @ApiOperation(value = "Cerca un Equipment per Id", notes = "Retorna un Equipment per Id")
    public ResponseEntity<Equipment> getEquipmentById(
            @PathVariable(value = "id") Long equipmentId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(equipmentService.getEquipmentById(equipmentId));
    }

    @PostMapping("/equipments")
    @ApiOperation(value = "Crea un Equipment", notes = "Crea un nou Equipment")
    public Equipment createEquipment(@Valid @RequestBody Equipment equipment) {
        return equipmentService.createEquipment(equipment);
    }

    @PutMapping("/equipments/{id}")
    @ApiOperation(value = "Actualitza un Equipment", notes = "Actualitza un Equipment per Id")
    public ResponseEntity<Equipment> updateEquipment(
            @PathVariable(value = "id") Long equipmentId,
            @Valid @RequestBody Equipment equipmentDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(equipmentService.updateEquipment(equipmentId, equipmentDetails));
    }

    @DeleteMapping("/equipments/{id}")
    @ApiOperation(value = "Esborra un Equipment", notes = "Esborra un Equipment per Id")
    public Map<String, Boolean> deleteEquipment(
            @PathVariable(value = "id") Long equipmentId) throws ResourceNotFoundException {

        return equipmentService.deleteEquipment(equipmentId);
    }


    //TODO: test
    @GetMapping("/equipments/room/{roomId}")
    @ApiOperation(value = "Cerca tots els Equipments per un Id d'una room", notes = "Retorna tots els Equipments per Id d'una room")
    public List<Equipment> getAllEquipmentByRoomId(
            @PathVariable(value = "roomId") Long roomId) throws ResourceNotFoundException {

        return equipmentService.getAllEquipmentByRoomId(roomId);
    }



}
