package com.tfm.reservas.controller;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import com.tfm.reservas.services.EventService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.Event;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Event")
public class EventController {

    @Autowired
    private EventService eventService;

    @GetMapping("/events")
    @ApiOperation(value = "Cerca tots els Events", notes = "Retorna tots els Events")
    public List<Event> getAllEvents() {
        return eventService.getAllEvents();
    }

    @GetMapping("/events/{id}")
    @ApiOperation(value = "Cerca un Event per Id", notes = "Retorna un Event per Id")
    public ResponseEntity<Event> getEventById(
            @PathVariable(value = "id") Long eventId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(eventService.getEventById(eventId));
    }

    @PostMapping("/events")
    @ApiOperation(value = "Crea un Event", notes = "Crea un nou Event")
    public Event createEvent(@Valid @RequestBody Event event) {
        return eventService.createEvent(event);
    }

    @PutMapping("/events/{id}")
    @ApiOperation(value = "Actualitza un Event", notes = "Actualitza un Event per Id")
    public ResponseEntity<Event> updateEvent(
            @PathVariable(value = "id") Long eventId,
            @Valid @RequestBody Event eventDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(eventService.updateEvent(eventId, eventDetails));
    }

    @DeleteMapping("/events/{id}")
    @ApiOperation(value = "Esborra un Event", notes = "Esborra un Event per Id")
    public Map<String, Boolean> deleteEvent(
            @PathVariable(value = "id") Long eventId) throws ResourceNotFoundException {

        return eventService.deleteEvent(eventId);
    }



}
