package com.tfm.reservas.controller;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.EventRoom;
import com.tfm.reservas.model.Room;
import com.tfm.reservas.services.EventRoomService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Event Room")
public class EventRoomController {

    @Autowired
    private EventRoomService eventRoomService;

    @GetMapping("/eventrooms")
    @ApiOperation(value = "Cerca tots els Event Room", notes = "Retorna tots els Event Room")
    public List<EventRoom> getAllEventRooms() {
        return eventRoomService.getAllEventRooms();
    }

    @GetMapping("/eventrooms/{id}")
    @ApiOperation(value = "Cerca un Event Room per Id", notes = "Retorna un Event Room per Id")
    public ResponseEntity<EventRoom> getEventRoomById(
            @PathVariable(value = "id") Long eventRoomId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(eventRoomService.getEventRoomById(eventRoomId));
    }

    @PostMapping("/eventrooms")
    @ApiOperation(value = "Crea un Event Room", notes = "Crea un nou Event Room")
    public EventRoom createEventRoom(@Valid @RequestBody EventRoom eventRoom) {
        return eventRoomService.createEventRoom(eventRoom);
    }

    @PutMapping("/eventrooms/{id}")
    @ApiOperation(value = "Actualitza un Event Room", notes = "Actualitza un Event Room per Id")
    public ResponseEntity<EventRoom> updateEventRoom(
            @PathVariable(value = "id") Long eventRoomId,
            @Valid @RequestBody EventRoom eventRoomDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(eventRoomService.updateEventRoom(eventRoomId, eventRoomDetails));
    }

    @DeleteMapping("/eventrooms/{id}")
    @ApiOperation(value = "Esborra un Event Room", notes = "Esborra un Event Room per Id")
    public Map<String, Boolean> deleteEventRoom(
            @PathVariable(value = "id") Long eventRoomId) throws ResourceNotFoundException {

        return eventRoomService.deleteEventRoom(eventRoomId);
    }

}
