package com.tfm.reservas.controller;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.Reservation;
import com.tfm.reservas.services.ReservationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Reservation")
public class ReservationController {

    @Autowired
    private ReservationService reservationService;

    @GetMapping("/reservations")
    @ApiOperation(value = "Cerca totes les Reservation", notes = "Retorna totes les Reservation")
    public List<Reservation> getAllReservations() {
        return reservationService.getAllReservations();
    }

    @GetMapping("/reservations/{id}")
    @ApiOperation(value = "Cerca una Reservation per Id", notes = "Retorna una Reservation per Id")
    public ResponseEntity<Reservation> getReservationById(
            @PathVariable(value = "id") Long reservationId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(reservationService.getReservationById(reservationId));
    }

    @PostMapping("/reservations")
    @ApiOperation(value = "Crea una Reservation", notes = "Crea una nova Reservation")
    public Reservation createReservation(@Valid @RequestBody Reservation reservation) {
        return reservationService.createReservation(reservation);
    }

    @PutMapping("/reservations/{id}")
    @ApiOperation(value = "Actualitza una Reservation", notes = "Actualitza una Reservation per Id")
    public ResponseEntity<Reservation> updateReservation(
            @PathVariable(value = "id") Long reservationId,
            @Valid @RequestBody Reservation reservationDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(reservationService.updateReservation(reservationId, reservationDetails));
    }

    @DeleteMapping("/reservations/{id}")
    @ApiOperation(value = "Esborra una Reservation", notes = "Esborra una Reservation per Id")
    public Map<String, Boolean> deleteReservation(
            @PathVariable(value = "id") Long reservationId) throws ResourceNotFoundException {

        return reservationService.deleteReservation(reservationId);
    }


    //TODO: test
    @GetMapping("/reservations/event/{eventId}")
    @ApiOperation(value = "Cerca totes les Reservation per un Id d'un event", notes = "Retorna totes les Reservation per Id d'event")
    public List<Reservation> getAllReservationByEventId(
            @PathVariable(value = "eventId") Long eventId) throws ResourceNotFoundException {

        return reservationService.getAllReservationsByEventId(eventId);

    }


}
