package com.tfm.reservas.controller;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.Room;
import com.tfm.reservas.services.RoomService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Room")
public class RoomController {

    @Autowired
    private RoomService roomService;

    @GetMapping("/rooms")
    @ApiOperation(value = "Cerca totes les Rooms", notes = "Retorna totes les Rooms")
    public List<Room> getAllRooms() {
        return roomService.getAllRooms();
    }

    @GetMapping("/rooms/{id}")
    @ApiOperation(value = "Cerca una Room per Id", notes = "Retorna una Room per Id")
    public ResponseEntity<Room> getRoomById(
            @PathVariable(value = "id") Long roomId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(roomService.getRoomById(roomId));
    }

    @PostMapping("/rooms")
    @ApiOperation(value = "Crea una Room", notes = "Crea una nova Room")
    public Room createRoom(@Valid @RequestBody Room room) {
        return roomService.createRoom(room);
    }

    @PutMapping("/rooms/{id}")
    @ApiOperation(value = "Actualitza una Room", notes = "Actualitza una Room per Id")
    public ResponseEntity<Room> updateRoom(
            @PathVariable(value = "id") Long roomId,
            @Valid @RequestBody Room roomDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(roomService.updateRoom(roomId, roomDetails));
    }

    @DeleteMapping("/rooms/{id}")
    @ApiOperation(value = "Esborra una Room", notes = "Esborra una Room per Id")
    public Map<String, Boolean> deleteRoom(
            @PathVariable(value = "id") Long roomId) throws ResourceNotFoundException {

        return roomService.deleteRoom(roomId);
    }


    //TODO: test
    @GetMapping("/rooms/event/{eventId}")
    @ApiOperation(value = "Cerca totes les Room per un Id d'un event", notes = "Retorna totes les Room per Id d'event")
    public List<Room> getAllRoomsByEventId(
            @PathVariable(value = "eventId") Long eventId) throws ResourceNotFoundException {
        return roomService.getAllRoomsByEventId(eventId);
    }



}
