package com.tfm.reservas.controller;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.RoomEquipment;
import com.tfm.reservas.services.RoomEquipmentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Room Equipment")
public class RoomEquipmentController {

    @Autowired
    private RoomEquipmentService roomEquipmentService;

    @GetMapping("/roomequipments")
    @ApiOperation(value = "Cerca tots els Room Equipment", notes = "Retorna tots els Room Equipment")
    public List<RoomEquipment> getAllRoomEquipments() {
        return roomEquipmentService.getAllRoomEquipments();
    }

    @GetMapping("/roomequipments/{id}")
    @ApiOperation(value = "Cerca un Room Equipment per Id", notes = "Retorna un Room Equipment per Id")
    public ResponseEntity<RoomEquipment> getRoomById(
            @PathVariable(value = "id") Long roomEquipmentId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(roomEquipmentService.getRoomEquipmentById(roomEquipmentId));
    }

    @PostMapping("/roomequipments")
    @ApiOperation(value = "Crea un Room Equipment", notes = "Crea un nou Room Equipment")
    public RoomEquipment createRoomEquipment(@Valid @RequestBody RoomEquipment roomEquipment) {
        return roomEquipmentService.createRoomEquipment(roomEquipment);
    }

    @PutMapping("/roomequipments/{id}")
    @ApiOperation(value = "Actualitza un Room Equipment", notes = "Actualitza un Room Equipment per Id")
    public ResponseEntity<RoomEquipment> updateRoomEquipment(
            @PathVariable(value = "id") Long roomEquipmentId,
            @Valid @RequestBody RoomEquipment roomEquipmentDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(roomEquipmentService.updateRoomEquipment(roomEquipmentId, roomEquipmentDetails));
    }

    @DeleteMapping("/roomequipments/{id}")
    @ApiOperation(value = "Esborra un Room Equipment", notes = "Esborra un Room Equipment per Id")
    public Map<String, Boolean> deleteRoomEquipment(
            @PathVariable(value = "id") Long roomEquipmentId) throws ResourceNotFoundException {

        return roomEquipmentService.deleteRoomEquipment(roomEquipmentId);
    }






}
