package com.tfm.reservas.controller;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.TypeReservation;
import com.tfm.reservas.services.TypeReservationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/api/v1")
@Api(value = "Type Reservation")
public class TypeReservationController {

    @Autowired
    private TypeReservationService typeReservationService;

    @GetMapping("/typereservations")
    @ApiOperation(value = "Cerca tots els Type Reservation", notes = "Retorna tots els Type Reservation")
    public List<TypeReservation> getAllTypeReservations() {
        return typeReservationService.getAllTypeReservations();
    }

    @GetMapping("/typereservations/{id}")
    @ApiOperation(value = "Cerca un Type Reservation per Id", notes = "Retorna un Type Reservation per Id")
    public ResponseEntity<TypeReservation> getTypeReservationById(
            @PathVariable(value = "id") Long typeReservationId) throws ResourceNotFoundException {
        return ResponseEntity.ok().body(typeReservationService.getTypeReservationById(typeReservationId));
    }

    @PostMapping("/typereservations")
    @ApiOperation(value = "Crea un Type Reservation", notes = "Crea un nou Type Reservation")
    public TypeReservation createTypeReservation(@Valid @RequestBody TypeReservation typeReservation) {
        return typeReservationService.createTypeReservation(typeReservation);
    }

    @PutMapping("/typereservations/{id}")
    @ApiOperation(value = "Actualitza un Type Reservation", notes = "Actualitza un Type Reservation per Id")
    public ResponseEntity<TypeReservation> updateTypeReservation(
            @PathVariable(value = "id") Long typeReservationId,
            @Valid @RequestBody TypeReservation typeReservationDetails) throws ResourceNotFoundException {

        return ResponseEntity.ok(typeReservationService.updateTypeReservation(typeReservationId, typeReservationDetails));
    }

    @DeleteMapping("/typereservations/{id}")
    @ApiOperation(value = "Esborra un Type Reservation", notes = "Esborra un Type Reservation per Id")
    public Map<String, Boolean> deleteTypeReservation(
            @PathVariable(value = "id") Long typeReservationId) throws ResourceNotFoundException {

        return typeReservationService.deleteTypeReservation(typeReservationId);
    }


}
