package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.Equipment;
import com.tfm.reservas.model.RoomEquipment;
import com.tfm.reservas.repository.EquipmentRepository;
import com.tfm.reservas.services.EquipmentService;
import com.tfm.reservas.services.RoomEquipmentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Transactional
public class EquipmentServiceImpl implements EquipmentService {

    @Autowired
    private EquipmentRepository equipmentRepository;

    @Autowired
    private RoomEquipmentService roomEquipmentService;

    @Override
    public List<Equipment> getAllEquipments() {
        return equipmentRepository.findAll();
    }

    @Override
    public Equipment getEquipmentById(Long equipmentId) throws ResourceNotFoundException {
        return equipmentRepository.findById(equipmentId).orElseThrow(() -> new ResourceNotFoundException("Equipo no encontrado :: "+ equipmentId));
    }

    @Override
    public Equipment createEquipment(Equipment equipment) {
        return equipmentRepository.save(equipment);
    }

    @Override
    public Equipment updateEquipment(Long equipmentId, Equipment equipmentDetails) throws ResourceNotFoundException {
        Equipment equipment = equipmentRepository.findById(equipmentId).orElseThrow(() -> new ResourceNotFoundException("Equipo no encontrado :: "+ equipmentId));

        equipment.setName(equipmentDetails.getName());
        equipment.setUpdatedBy(equipmentDetails.getUpdatedBy());
        equipment.setUpdatedAt(new Date());
        return equipmentRepository.save(equipment);
    }

    @Override
    public Map<String, Boolean> deleteEquipment(Long equipmentId) throws ResourceNotFoundException {
        Equipment equipment = equipmentRepository.findById(equipmentId).orElseThrow(() -> new ResourceNotFoundException("Equipo no encontrado :: "+ equipmentId));

        equipmentRepository.delete(equipment);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }

    @Override
    public List<Equipment> getAllEquipmentByRoomId(Long roomId) throws ResourceNotFoundException {

        List<RoomEquipment> roomEquipmentsList = roomEquipmentService.getAllRoomEquipments();
        List<Equipment> equipmentResultList = new ArrayList<>();
        for (RoomEquipment roomEquipment : roomEquipmentsList) {
            if (roomEquipment.getRoom().getId() == roomId){
                Equipment equipment = equipmentRepository.findById(roomEquipment.getEquipment().getId()).orElseThrow(() -> new ResourceNotFoundException("Equipment no encontrado :: "+ roomEquipment.getEquipment().getId()));
                equipmentResultList.add(equipment);
            }
        }

        return equipmentResultList;
    }

}
