package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.EventRoom;
import com.tfm.reservas.repository.EventRoomRepository;
import com.tfm.reservas.services.EventRoomService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class EventRoomServiceImpl implements EventRoomService {

    @Autowired
    private EventRoomRepository eventRoomRepository;

    @Override
    public List<EventRoom> getAllEventRooms() {
        return eventRoomRepository.findAll();
    }

    @Override
    public EventRoom getEventRoomById(Long eventRoomId) throws ResourceNotFoundException {
        return eventRoomRepository.findById(eventRoomId).orElseThrow(() -> new ResourceNotFoundException("EventRoom no encontrado :: "+ eventRoomId));
    }

    @Override
    public EventRoom createEventRoom(EventRoom eventRoom) {
        return eventRoomRepository.save(eventRoom);
    }

    @Override
    public EventRoom updateEventRoom(Long eventRoomId, EventRoom eventRoomDetails) throws ResourceNotFoundException {
        EventRoom eventRoom = eventRoomRepository.findById(eventRoomId).orElseThrow(() -> new ResourceNotFoundException("EventRoom no encontrado :: "+ eventRoomId));

        eventRoom.setEvent(eventRoomDetails.getEvent());
        eventRoom.setRoom(eventRoomDetails.getRoom());
        eventRoom.setUpdatedBy(eventRoomDetails.getUpdatedBy());
        eventRoom.setUpdatedAt(new Date());
        return eventRoomRepository.save(eventRoom);
    }

    @Override
    public Map<String, Boolean> deleteEventRoom(Long eventRoomId) throws ResourceNotFoundException {
        EventRoom eventRoom = eventRoomRepository.findById(eventRoomId).orElseThrow(() -> new ResourceNotFoundException("EventRoom no encontrado :: "+ eventRoomId));

        eventRoomRepository.delete(eventRoom);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }

}
