package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.Event;
import com.tfm.reservas.repository.EventRepository;
import com.tfm.reservas.services.EventService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class EventServiceImpl implements EventService {

    @Autowired
    private EventRepository eventRepository;

    @Override
    public List<Event> getAllEvents() {
        return eventRepository.findAll();
    }

    @Override
    public Event getEventById(Long eventId) throws ResourceNotFoundException {
        return eventRepository.findById(eventId).orElseThrow(() -> new ResourceNotFoundException("Evento no encontrado :: "+ eventId));
    }

    @Override
    public Event createEvent(Event event) {
        return eventRepository.save(event);
    }

    @Override
    public Event updateEvent(Long eventId, Event eventDetails) throws ResourceNotFoundException {
        Event event = eventRepository.findById(eventId).orElseThrow(() -> new ResourceNotFoundException("Evento no encontrado :: "+ eventId));

        event.setName(eventDetails.getName());
        event.setDescription(eventDetails.getDescription());
        event.setUpdatedBy(eventDetails.getUpdatedBy());
        event.setUpdatedAt(new Date());
        return eventRepository.save(event);
    }

    @Override
    public Map<String, Boolean> deleteEvent(Long eventId) throws ResourceNotFoundException {
        Event event = eventRepository.findById(eventId).orElseThrow(() -> new ResourceNotFoundException("Evento no encontrado :: "+ eventId));

        eventRepository.delete(event);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }


}
