package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.EventRoom;
import com.tfm.reservas.model.Reservation;
import com.tfm.reservas.model.Room;
import com.tfm.reservas.repository.ReservationRepository;
import com.tfm.reservas.repository.RoomRepository;
import com.tfm.reservas.services.EventRoomService;
import com.tfm.reservas.services.ReservationService;
import com.tfm.reservas.services.RoomService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Transactional
public class ReservationServiceImpl implements ReservationService {

    @Autowired
    private ReservationRepository reservationRepository;

    @Autowired
    private EventRoomService eventRoomService;

    @Autowired
    private RoomService roomService;


    @Override
    public List<Reservation> getAllReservations() {
        return reservationRepository.findAll();
    }

    @Override
    public Reservation getReservationById(Long reservationId) throws ResourceNotFoundException {
        return reservationRepository.findById(reservationId).orElseThrow(() -> new ResourceNotFoundException("Reserva no encontrada :: "+ reservationId));
    }

    @Override
    public Reservation createReservation(Reservation reservation) {
        return reservationRepository.save(reservation);
    }

    @Override
    public Reservation updateReservation(Long reservationId, Reservation reservationDetails) throws ResourceNotFoundException {
        Reservation reservation = reservationRepository.findById(reservationId).orElseThrow(() -> new ResourceNotFoundException("Reserva no encontrada :: "+ reservationId));

        reservation.setIdClient(reservationDetails.getIdClient());
        reservation.setIdRoom(reservationDetails.getIdRoom());
        reservation.setDate(reservationDetails.getDate());
        reservation.setTypeReservation(reservationDetails.getTypeReservation());

        reservation.setUpdatedBy(reservationDetails.getUpdatedBy());
        reservation.setUpdatedAt(new Date());
        return reservationRepository.save(reservation);
    }

    @Override
    public Map<String, Boolean> deleteReservation(Long reservationId) throws ResourceNotFoundException {
        Reservation reservation = reservationRepository.findById(reservationId).orElseThrow(() -> new ResourceNotFoundException("Reserva no encontrada :: "+ reservationId));

        reservationRepository.delete(reservation);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }


    @Override
    public List<Reservation> getAllReservationsByEventId(Long eventId) throws ResourceNotFoundException {

        List<Room> roomListByEventId = roomService.getAllRoomsByEventId(eventId);
        List<Reservation> reservationList = reservationRepository.findAll();
        List<Reservation> reservationResultList = new ArrayList<>();

        for (Reservation reservation : reservationList) {
            for (Room room : roomListByEventId) {
                if (reservation.getIdRoom().getId() == room.getId()) {
                    reservationResultList.add(reservation);
                }
            }
        }

        return reservationResultList;
    }


}
