package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.RoomEquipment;
import com.tfm.reservas.repository.RoomEquipmentRepository;
import com.tfm.reservas.services.RoomEquipmentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class RoomEquipmentServiceImpl implements RoomEquipmentService {

    @Autowired
    private RoomEquipmentRepository roomEquipmentRepository;

    @Override
    public List<RoomEquipment> getAllRoomEquipments() {
        return roomEquipmentRepository.findAll();
    }

    @Override
    public RoomEquipment getRoomEquipmentById(Long roomEquipmentId) throws ResourceNotFoundException {
        return roomEquipmentRepository.findById(roomEquipmentId).orElseThrow(() -> new ResourceNotFoundException("RoomEquipment no encontrado :: "+ roomEquipmentId));
    }

    @Override
    public RoomEquipment createRoomEquipment(RoomEquipment roomEquipment) {
        return roomEquipmentRepository.save(roomEquipment);
    }

    @Override
    public RoomEquipment updateRoomEquipment(Long roomEquipmentId, RoomEquipment roomEquipmentDetails) throws ResourceNotFoundException {
        RoomEquipment roomEquipment = roomEquipmentRepository.findById(roomEquipmentId).orElseThrow(() -> new ResourceNotFoundException("RoomEquipment no encontrado :: "+ roomEquipmentId));

        roomEquipment.setRoom(roomEquipmentDetails.getRoom());
        roomEquipment.setEquipment(roomEquipmentDetails.getEquipment());
        roomEquipment.setQuantity(roomEquipmentDetails.getQuantity());

        roomEquipment.setUpdatedBy(roomEquipmentDetails.getUpdatedBy());
        roomEquipment.setUpdatedAt(new Date());
        return roomEquipmentRepository.save(roomEquipment);
    }

    @Override
    public Map<String, Boolean> deleteRoomEquipment(Long roomEquipmentId) throws ResourceNotFoundException {
        RoomEquipment roomEquipment = roomEquipmentRepository.findById(roomEquipmentId).orElseThrow(() -> new ResourceNotFoundException("RoomEquipment no encontrado :: "+ roomEquipmentId));

        roomEquipmentRepository.delete(roomEquipment);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }


}
