package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.EventRoom;
import com.tfm.reservas.model.Room;
import com.tfm.reservas.repository.RoomRepository;
import com.tfm.reservas.services.EventRoomService;
import com.tfm.reservas.services.RoomService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Transactional
public class RoomServiceImpl implements RoomService {

    @Autowired
    private RoomRepository roomRepository;

    @Autowired
    private EventRoomService eventRoomService;

    @Override
    public List<Room> getAllRooms() {
        return roomRepository.findAll();
    }

    @Override
    public Room getRoomById(Long roomId) throws ResourceNotFoundException {
        return roomRepository.findById(roomId).orElseThrow(() -> new ResourceNotFoundException("Room no encontrada :: "+ roomId));
    }

    @Override
    public Room createRoom(Room room) {
        return roomRepository.save(room);
    }

    @Override
    public Room updateRoom(Long roomId, Room roomDetails) throws ResourceNotFoundException {
        Room room = roomRepository.findById(roomId).orElseThrow(() -> new ResourceNotFoundException("Room no encontrada :: "+ roomId));

        room.setName(roomDetails.getName());
        room.setCapacity(roomDetails.getCapacity());
        room.setUpdatedBy(roomDetails.getUpdatedBy());
        room.setUpdatedAt(new Date());
        return roomRepository.save(room);
    }

    @Override
    public Map<String, Boolean> deleteRoom(Long roomId) throws ResourceNotFoundException {
        Room room = roomRepository.findById(roomId).orElseThrow(() -> new ResourceNotFoundException("Room no encontrada :: "+ roomId));

        roomRepository.delete(room);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }



    @Override
    public List<Room> getAllRoomsByEventId(Long eventId) throws ResourceNotFoundException  {

        List<EventRoom> eventRoomList = eventRoomService.getAllEventRooms();
        List<Room> roomResultList = new ArrayList<>();
        for (EventRoom eventRoom : eventRoomList) {
            if (eventRoom.getEvent().getId() == eventId){
                Room room = roomRepository.findById(eventRoom.getRoom().getId()).orElseThrow(() -> new ResourceNotFoundException("Room no encontrada :: "+ eventRoom.getRoom().getId()));;
                roomResultList.add(room);
            }
        }

        return roomResultList;
    }
}
