package com.tfm.reservas.services.impl;

import com.tfm.reservas.exception.ResourceNotFoundException;
import com.tfm.reservas.model.TypeReservation;
import com.tfm.reservas.repository.TypeReservationRepository;
import com.tfm.reservas.services.TypeReservationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class TypeReservationServiceImpl implements TypeReservationService {

    @Autowired
    private TypeReservationRepository typeReservationRepository;

    @Override
    public List<TypeReservation> getAllTypeReservations() {
        return typeReservationRepository.findAll();
    }

    @Override
    public TypeReservation getTypeReservationById(Long typeReservationId) throws ResourceNotFoundException {
        return typeReservationRepository.findById(typeReservationId).orElseThrow(() -> new ResourceNotFoundException("TypeReservation no encontrado :: "+ typeReservationId));
    }

    @Override
    public TypeReservation createTypeReservation(TypeReservation typeReservation) {
        return typeReservationRepository.save(typeReservation);
    }

    @Override
    public TypeReservation updateTypeReservation(Long typeReservationId, TypeReservation typeReservationDetails) throws ResourceNotFoundException {
        TypeReservation typeReservation = typeReservationRepository.findById(typeReservationId).orElseThrow(() -> new ResourceNotFoundException("TypeReservation no encontrado :: "+ typeReservationId));

        typeReservation.setDescription(typeReservationDetails.getDescription());
        typeReservation.setUpdatedBy(typeReservationDetails.getUpdatedBy());
        typeReservation.setUpdatedAt(new Date());
        return typeReservationRepository.save(typeReservation);
    }

    @Override
    public Map<String, Boolean> deleteTypeReservation(Long typeReservationId) throws ResourceNotFoundException {
        TypeReservation typeReservation = typeReservationRepository.findById(typeReservationId).orElseThrow(() -> new ResourceNotFoundException("TypeReservation no encontrado :: "+ typeReservationId));

        typeReservationRepository.delete(typeReservation);
        Map<String, Boolean> response = new HashMap<>();
        response.put("deleted", Boolean.TRUE);
        return response;
    }



}
