package com.tfm.reservas;

import com.tfm.reservas.model.Equipment;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.HttpClientErrorException;

import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class EquipmentTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllEquipments() {
        HttpHeaders headers = new HttpHeaders();
        //headers.add("Authorization", "ZGhpZGFsZ286ZGhpZGFsZ28=");
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/equipments",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetEquipmentById() {
        Equipment equipment = restTemplate.getForObject(getRootUrl() + "/equipments/1", Equipment.class);
        System.out.println(equipment.getName());
        assertNotNull(equipment);
    }

    @Test
    public void testCreateEquipment() {
        Equipment equipment = new Equipment();
        equipment.setName("Equipment Demo");
        equipment.setCreatedBy("admin_demo");
        equipment.setUpdatedBy("admin_demo");

        /*
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<Equipment> entity = new HttpEntity<Equipment>(null, headers);
        ResponseEntity<Equipment> postResponse;
        postResponse = restTemplate.exchange(getRootUrl() + "/equipments", HttpMethod.POST, entity, Equipment.class);
        */
        ResponseEntity<Equipment> postResponse = restTemplate.postForEntity(getRootUrl() + "/equipments", equipment, Equipment.class);

        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testUpdatePost() {
        int id = 1;
        Equipment equipment = restTemplate.getForObject(getRootUrl() + "/equipments/" + id, Equipment.class);
        equipment.setName("Equipment Demo 2");

        restTemplate.put(getRootUrl() + "/equipments/" + id, equipment);

        Equipment updatedEquipment = restTemplate.getForObject(getRootUrl() + "/equipments/" + id, Equipment.class);
        assertNotNull(updatedEquipment);
    }

    @Test
    public void testDeleteEquipment() {
        int id = 2;
        Equipment equipment = restTemplate.getForObject(getRootUrl() + "/equipments/" + id, Equipment.class);
        assertNotNull(equipment);

        restTemplate.delete(getRootUrl() + "/equipments/" + id);

        try {
            equipment = restTemplate.getForObject(getRootUrl() + "/equipments/" + id, Equipment.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }

}
