package com.tfm.reservas;

import com.tfm.reservas.model.Event;
import com.tfm.reservas.model.EventRoom;
import com.tfm.reservas.model.Room;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.HttpClientErrorException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class EventRoomTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllEventRooms() {
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/eventrooms",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetEventRoomById() {
        EventRoom eventRoom = restTemplate.getForObject(getRootUrl() + "/eventrooms/1", EventRoom.class);
        System.out.println(eventRoom.getId());
        assertNotNull(eventRoom);
    }

    @Test
    public void testCreateEventRoom() {
        EventRoom eventRoom = new EventRoom();
        eventRoom.setEvent(createEvent());
        eventRoom.setRoom(createRoom());

        eventRoom.setCreatedBy("admin_demo");
        eventRoom.setUpdatedBy("admin_demo");

        ResponseEntity<EventRoom> postResponse = restTemplate.postForEntity(getRootUrl() + "/eventrooms", eventRoom, EventRoom.class);
        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testUpdatePost() {
        int id = 1;
        EventRoom eventRoom = restTemplate.getForObject(getRootUrl() + "/eventrooms/" + id, EventRoom.class);
        eventRoom.setRoom(createRoomUpdate());

        restTemplate.put(getRootUrl() + "/eventrooms/" + id, eventRoom);

        EventRoom updatedEventRoom = restTemplate.getForObject(getRootUrl() + "/eventrooms/" + id, EventRoom.class);
        assertNotNull(updatedEventRoom);
    }

    @Test
    public void testDeleteEventRoom() {
        int id = 2;
        EventRoom eventRoom = restTemplate.getForObject(getRootUrl() + "/eventrooms/" + id, EventRoom.class);
        assertNotNull(eventRoom);

        restTemplate.delete(getRootUrl() + "/eventrooms/" + id);

        try {
            eventRoom = restTemplate.getForObject(getRootUrl() + "/eventrooms/" + id, EventRoom.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }


    public Event createEvent(){
        Event event = new Event();
        event.setName("Event Demo");
        event.setDescription("Descripción Evento Demo");
        event.setCreatedBy("admin_demo");
        event.setUpdatedBy("admin_demo");
        return event;
    }

    public Room createRoom(){
        Room room = new Room();
        room.setName("Room Demo");
        room.setCapacity(12L);
        room.setCreatedBy("admin_demo");
        room.setUpdatedBy("admin_demo");
        return room;
    }

    public Room createRoomUpdate(){
        Room room = new Room();
        room.setName("Room Demo Update");
        room.setCapacity(24L);
        room.setCreatedBy("admin_demo");
        room.setUpdatedBy("admin_demo");
        return room;
    }


}
