package com.tfm.reservas;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.HttpClientErrorException;

import com.tfm.reservas.model.Event;


@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class EventTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllEvents() {
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/events",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetEventById() {
        Event event = restTemplate.getForObject(getRootUrl() + "/events/1", Event.class);
        System.out.println(event.getName());
        assertNotNull(event);
    }

    @Test
    public void testCreateEvent() {
        Event event = new Event();
        event.setName("Evento Demo");
        event.setDescription("Descripción Evento Demo");
        event.setCreatedBy("admin_demo");
        event.setUpdatedBy("admin_demo");

        ResponseEntity<Event> postResponse = restTemplate.postForEntity(getRootUrl() + "/events", event, Event.class);
        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testUpdatePost() {
        int id = 1;
        Event event = restTemplate.getForObject(getRootUrl() + "/events/" + id, Event.class);
        event.setName("Evento Demo 2");
        event.setDescription("Descripción Evento Demo 2");

        restTemplate.put(getRootUrl() + "/events/" + id, event);

        Event updatedEvent = restTemplate.getForObject(getRootUrl() + "/events/" + id, Event.class);
        assertNotNull(updatedEvent);
    }

    @Test
    public void testDeleteEvent() {
        int id = 2;
        Event event = restTemplate.getForObject(getRootUrl() + "/events/" + id, Event.class);
        assertNotNull(event);

        restTemplate.delete(getRootUrl() + "/events/" + id);

        try {
            event = restTemplate.getForObject(getRootUrl() + "/events/" + id, Event.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }

}
