package com.tfm.reservas;

import com.tfm.reservas.model.Reservation;
import com.tfm.reservas.model.Room;
import com.tfm.reservas.model.TypeReservation;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.HttpClientErrorException;

import java.lang.reflect.Type;
import java.util.Date;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)

public class ReservationTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllReservations() {
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/reservations",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetReservationById() {
        Reservation reservation = restTemplate.getForObject(getRootUrl() + "/reservations/1", Reservation.class);
        System.out.println(reservation.getIdClient());
        assertNotNull(reservation);
    }

    @Test
    public void testCreateReservation() {
        Reservation reservation = new Reservation();
        reservation.setIdClient("12345678");
        reservation.setIdRoom(createRoom());
        reservation.setDate(new Date(2019, 10, 17, 12,01,55));
        reservation.setTypeReservation(createTypeReservation());
        reservation.setCreatedBy("admin_demo");
        reservation.setUpdatedBy("admin_demo");

        ResponseEntity<Reservation> postResponse = restTemplate.postForEntity(getRootUrl() + "/reservations", reservation, Reservation.class);
        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testDeleteReservation() {
        int id = 2;
        Reservation reservation = restTemplate.getForObject(getRootUrl() + "/reservations/" + id, Reservation.class);
        assertNotNull(reservation);

        restTemplate.delete(getRootUrl() + "/reservations/" + id);

        try {
            reservation = restTemplate.getForObject(getRootUrl() + "/reservations/" + id, Reservation.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }


    public Room createRoom(){
        Room room = new Room();
        room.setName("Room Demo");
        room.setCapacity(12L);
        room.setCreatedBy("admin_demo");
        room.setUpdatedBy("admin_demo");
        return room;
    }

    public TypeReservation createTypeReservation() {
        TypeReservation typeReservation = new TypeReservation();
        typeReservation.setDescription("TypeReservation Demo");
        typeReservation.setCreatedBy("admin_demo");
        typeReservation.setUpdatedBy("admin_demo");
        return typeReservation;
    }

}
