package com.tfm.reservas;

import com.tfm.reservas.model.Equipment;
import com.tfm.reservas.model.Room;
import com.tfm.reservas.model.RoomEquipment;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.HttpClientErrorException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class RoomEquipmentTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllRoomEquipments() {
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/roomequipments",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetRoomEquipmentById() {
        RoomEquipment roomEquipment = restTemplate.getForObject(getRootUrl() + "/roomequipments/1", RoomEquipment.class);
        System.out.println(roomEquipment.getQuantity());
        assertNotNull(roomEquipment);
    }

    @Test
    public void testCreateRoomEquipment() {
        RoomEquipment roomEquipment = new RoomEquipment();
        roomEquipment.setRoom(createRoom());
        roomEquipment.setEquipment(createEquipment());
        roomEquipment.setQuantity(6);
        roomEquipment.setCreatedBy("admin_demo");
        roomEquipment.setUpdatedBy("admin_demo");

        ResponseEntity<RoomEquipment> postResponse = restTemplate.postForEntity(getRootUrl() + "/roomequipments", roomEquipment, RoomEquipment.class);
        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testUpdatePost() {
        int id = 1;
        RoomEquipment roomEquipment = restTemplate.getForObject(getRootUrl() + "/roomequipments/" + id, RoomEquipment.class);
        roomEquipment.setQuantity(12);

        restTemplate.put(getRootUrl() + "/roomequipments/" + id, roomEquipment);

        RoomEquipment updatedRoomEquipment = restTemplate.getForObject(getRootUrl() + "/roomequipments/" + id, RoomEquipment.class);
        assertNotNull(updatedRoomEquipment);
    }

    @Test
    public void testDeleteEventRoomEquipment() {
        int id = 2;
        RoomEquipment roomEquipment = restTemplate.getForObject(getRootUrl() + "/roomequipments/" + id, RoomEquipment.class);
        assertNotNull(roomEquipment);

        restTemplate.delete(getRootUrl() + "/roomequipments/" + id);

        try {
            roomEquipment = restTemplate.getForObject(getRootUrl() + "/roomequipments/" + id, RoomEquipment.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }


    public Room createRoom(){
        Room room = new Room();
        room.setName("Room Demo");
        room.setCapacity(12L);
        room.setCreatedBy("admin_demo");
        room.setUpdatedBy("admin_demo");
        return room;
    }

    public Equipment createEquipment() {
        Equipment equipment = new Equipment();
        equipment.setName("Equipment Demo");
        equipment.setCreatedBy("admin_demo");
        equipment.setUpdatedBy("admin_demo");
        return equipment;
    }


}
