package com.tfm.reservas;

import com.tfm.reservas.model.Room;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.HttpClientErrorException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class RoomTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllRooms() {
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/rooms",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetRoomById() {
        Room room = restTemplate.getForObject(getRootUrl() + "/rooms/1", Room.class);
        System.out.println(room.getName());
        assertNotNull(room);
    }

    @Test
    public void testCreateRoom() {
        Room room = new Room();
        room.setName("Room Demo");
        room.setCapacity(12L);
        room.setCreatedBy("admin_demo");
        room.setUpdatedBy("admin_demo");

        ResponseEntity<Room> postResponse = restTemplate.postForEntity(getRootUrl() + "/rooms", room, Room.class);
        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testUpdatePost() {
        int id = 1;
        Room room = restTemplate.getForObject(getRootUrl() + "/rooms/" + id, Room.class);
        room.setName("Room Demo 2");

        restTemplate.put(getRootUrl() + "/rooms/" + id, room);

        Room updatedRoom = restTemplate.getForObject(getRootUrl() + "/rooms/" + id, Room.class);
        assertNotNull(updatedRoom);
    }

    @Test
    public void testDeleteEquipment() {
        int id = 2;
        Room room = restTemplate.getForObject(getRootUrl() + "/rooms/" + id, Room.class);
        assertNotNull(room);

        restTemplate.delete(getRootUrl() + "/rooms/" + id);

        try {
            room = restTemplate.getForObject(getRootUrl() + "/rooms/" + id, Room.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }




}
