package com.tfm.reservas;

import com.tfm.reservas.model.TypeReservation;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.http.*;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.client.HttpClientErrorException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = ReservasApplication.class, webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
public class TypeReservationTests {

    @Autowired
    private TestRestTemplate restTemplate = new TestRestTemplate("dhidalgo","dhidalgo");

    @LocalServerPort
    private int port;

    @Value("${host}")
    private String host;

    private String getRootUrl() {
        return host + ":" + port;
    }

    @Test
    public void contextLoads() {
    }

    @Test
    public void testGetAllTypeReservations() {
        HttpHeaders headers = new HttpHeaders();
        HttpEntity<String> entity = new HttpEntity<String>(null, headers);

        ResponseEntity<String> response = restTemplate.exchange(getRootUrl() + "/typereservations",
                HttpMethod.GET, entity, String.class);

        assertNotNull(response.getBody());
    }

    @Test
    public void testGetTypeReservationById() {
        TypeReservation typeReservation = restTemplate.getForObject(getRootUrl() + "/typereservations/1", TypeReservation.class);
        System.out.println(typeReservation.getDescription());
        assertNotNull(typeReservation);
    }

    @Test
    public void testCreateTypeReservation() {
        TypeReservation typeReservation = new TypeReservation();
        typeReservation.setDescription("Descripción TypeReservation Demo");
        typeReservation.setCreatedBy("admin_demo");
        typeReservation.setUpdatedBy("admin_demo");

        ResponseEntity<TypeReservation> postResponse = restTemplate.postForEntity(getRootUrl() + "/typereservations", typeReservation, TypeReservation.class);
        assertNotNull(postResponse);
        assertNotNull(postResponse.getBody());
    }

    @Test
    public void testUpdatePost() {
        int id = 1;
        TypeReservation typeReservation = restTemplate.getForObject(getRootUrl() + "/typereservations/" + id, TypeReservation.class);
        typeReservation.setDescription("Descripción TypeReservation Demo 2");

        restTemplate.put(getRootUrl() + "/typereservations/" + id, typeReservation);

        TypeReservation updatedTypeReservation = restTemplate.getForObject(getRootUrl() + "/typereservations/" + id, TypeReservation.class);
        assertNotNull(updatedTypeReservation);
    }

    @Test
    public void testDeleteTypeReservation() {
        int id = 2;
        TypeReservation typeReservation = restTemplate.getForObject(getRootUrl() + "/typereservations/" + id, TypeReservation.class);
        assertNotNull(typeReservation);

        restTemplate.delete(getRootUrl() + "/typereservations/" + id);

        try {
            typeReservation = restTemplate.getForObject(getRootUrl() + "/typereservations/" + id, TypeReservation.class);
        } catch (final HttpClientErrorException e) {
            assertEquals(e.getStatusCode(), HttpStatus.NOT_FOUND);
        }
    }

}
